# copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu

set rcsId {$Id: route.tcl,v 1.5 1998/12/20 16:47:06 jfontain Exp $}

package provide route 1.0

package require network

namespace eval route {

    array set data {
        updates 0
        0,label interface 0,type ascii 0,message {networking interface}
        1,label destination 1,type dictionary 1,message {destination network or host, or default}
        2,label gateway 2,type dictionary 2,message {gateway address, if any}
        3,label flags 3,type ascii 3,message {D: dynamic, G: gateway, H: host, M: modified, R: reinstate, U: up, !: reject}
        4,label mask 4,type dictionary 4,message {network mask (empty for default route)}
        5,label references 5,type integer 5,message {number of references to the route}
        6,label use 6,type integer 6,message {count of lookups for the route}
        7,label metric 7,type integer 7,message {distance (in hops) to the target}
        pollTimes {20 10 30 60 120 300 600}
        sort {1 increasing}
        helpText {
This is a view of network routes for the system (tested on IPv4 only).
Networks and hosts addresses are looked up and converted to official names when possible.
        }
    }

    set file [open /proc/net/route]                                                     ;# keep the file open for better performance

    set nextIndex 0

    proc update {} {
        variable file
        variable index
        variable nextIndex
        variable data

        seek $file 0                                                                                ;# rewind before retrieving data
        gets $file                                                                                               ;# skip header line
        while {[gets $file line]>=0} {
            scan $line {%s %2x%2x%2x%2x %2x%2x%2x%2x %x %u %u %u %2x%2x%2x%2x}\
                interface destination(3) destination(2) destination(1) destination(0)\
                gateway(3) gateway(2) gateway(1) gateway(0) flags references use metric mask(3) mask(2) mask(1) mask(0)
            set address $destination(0).$destination(1).$destination(2).$destination(3)
            if {[catch {set index($address)} row]} {                                                                    ;# new entry
                set row [set index($address) $nextIndex]
                incr nextIndex
            }
            set data($row,0) $interface
            if {[string compare $address 0.0.0.0]==0} {                                                             ;# default route
                set data($row,1) default
            } elseif {[catch {set data($row,1) [network::networkfromaddress $address]}]} {                        ;# try name lookup
                set data($row,1) $address
            }
            set data($row,2) $gateway(0).$gateway(1).$gateway(2).$gateway(3)
            if {[string compare $data($row,2) 0.0.0.0]==0} {
                set data($row,2) {}                                                                                    ;# no gateway
            } else {
                catch {set data($row,2) [network::hostfromaddress $data($row,2)]}                                 ;# try name lookup
            }
            set data($row,3) [network::routeletterflags $flags]
            set data($row,4) $mask(0).$mask(1).$mask(2).$mask(3)
            if {[string compare $data($row,4) 0.0.0.0]==0} {
                set data($row,4) {}                                                                                       ;# no mask
            }
            set data($row,5) $references
            set data($row,6) $use
            set data($row,7) $metric
            set current($address) {}
        }
        foreach address [array names index] {                                                         ;# cleanup disappeared entries
            if {[info exists current($address)]} continue
            set row $index($address)
            unset index($address) data($row,0) data($row,1) data($row,2) data($row,3)
        }
        incr data(updates)
    }
}
